import numpy as np
from numpy.linalg import norm
from math import pi, sqrt,exp
import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle
from scipy import stats
import matplotlib.pyplot as plt




#Erzeuge Poisson-Prozess
def Punkte(gamma,a,b,R):
    #Beachte: die "+-R" ist der Einfluss von ausserhalb des Fensters
    Z = np.random.uniform(low=(-2*R,-R), high=(a,b+R), size=(np.random.poisson(gamma*(a+2*R)*(b+2*R)),2))
    #Gib die lexikographisch geordnete Liste aus
    return(Z[np.lexsort(np.rot90(Z))])



#Pruefe ob positive TP ueberdeckt werden
def TestPos(Z,a,b,R):
    N = len(Z)
    Loeschen = []
    for i in range(N-1):
        j = i + 1 
        while j <= N-1 and Z[j][0]-Z[i][0] < R:
            if Z[j][1] > Z[i][1]:
                if norm(Z[j]-[0,R]-Z[i],2) < R:
                    #Falls ja wird der j-te Tangentenpunkt überdeckt
                    Loeschen.append(j)
                j = j+1
            else:
                if norm(Z[j]-Z[i]+[0,R],2) < R:
                    #Falls ja wird der i-te Tangentenpunkt überdeckt
                    Loeschen.append(i)
                j = j+1
    TP = np.delete(Z,Loeschen,0) + [R,-R]

    #Langsamer
    #Loeschen = []
    #for i in range(len(TP)):
    #    if TP[i][0] < 0 or TP[i][0] > a or TP[i][1] < 0 or TP[i][1] > b:
    #        Loeschen.append(i)
    #return(np.delete(TP,Loeschen,0))

    #Schneller
    return(np.delete(TP,[i for i,x in enumerate(TP) if x[0] < 0 or x[0] > a or x[1] < 0 or x[1] > b],0))






#Sichtbare negative Tangentenpunkte bestimmen
def TestNeg(Z,a,b,R):
    N = len(Z)
    TP = []
    #Finde mögliche TP    
    for i in range(N-1):
        j = i + 1 
        while j <= N-1 and Z[j][0]-Z[i][0] < 2*R:
            #Dazu ist die notwendige Bedigung
            if norm(Z[j]-Z[i],2) < 2*R and norm(Z[j]-Z[i]-[0,R] ,2) > R and norm(Z[j]-Z[i]+[0,R] ,2) > R:
                #Welche Koordinaten hat dann der TP?
                c = norm(Z[j]-Z[i] ,2)
                x = (R**2 + c**2 - R**2)/(2*c)
                TPneg = Z[i]+ np.array([R,0]) + x/c*(Z[j] - Z[i]) - ( R**2 - x**2 )**(1/2)/c *np.array([Z[i][1]-Z[j][1] , Z[j][0]-Z[i][0]])


                #Werden diese TP ueberdeckt?
                k = 0
                while k <= N-1 and Z[k][0]-Z[j][0] < 2*R:
                    if k != j and k != i and norm(TPneg-Z[k]-[R,0],2) < R:
                        k = N + 3
                    else:
                        k = k + 1

                if k <= N+1:
                    TP.append(TPneg)
                      

            j = j+1
        

    #Langsamer
    #Loeschen = []
    #for i in range(len(TP)):
    #    if TP[i][0] < 0 or TP[i][0] > a or TP[i][1] < 0 or TP[i][1] > b:
    #        Loeschen.append(i)            
    #return(np.delete(TP,Loeschen,0))

    #Schneller
    return(np.delete(TP,[i for i,x in enumerate(TP) if x[0] < 0 or x[0] > a or x[1] < 0 or x[1] > b],0))





#Definiere die Kernfunktion
def K(x):
    z = norm(x,2)
    if z < 1:
        Wert = 1-z**2
    else:
        Wert = 0
    return(Wert)


#Definiere die Funktion zum Flaecheberechnen
def Area(u):
    return(u[0]*u[1])



#Berechne die EPC
def Chi(gamma,a,b,R):
    Z = Punkte(gamma,a,b,R)
    TPp = TestPos(Z,a,b,R)
    TPn = TestNeg(Z,a,b,R)
    Chi = len(TPp) - len(TPn)
    folge = (a*b)**(1/6)
    sigma = sum([K((x-y)/folge)/Area([a,b]-abs(x-y)) for x in TPp for y in TPp]) + sum([K((x-y)/folge)/Area([a,b]-abs(x-y)) for x in TPn for y in TPn]) - 2*sum([K((x-y)/folge)/Area([a,b]-abs(x-y)) for x in TPp for y in TPn]) - folge**2*pi/2*(Chi/(a*b))**2 
    Normal = sqrt(a*b/sigma)*(Chi/(a*b)-(gamma*exp(-gamma*pi*R**2)*(1-gamma*pi*R**2)))
    return(Normal)








#Testplot
def Testplot(gamma,a,b,R):
    #Erstelle Liste mit Punkten
    Z = Punkte(gamma,a,b,R)
    TPp = TestPos(Z,a,b,R)
    TPn = TestNeg(Z,a,b,R)
    #Fummle die Liste auseinander
    xpunkte = [x[0] for x in Z]
    ypunkte = [x[1] for x in Z]
    #Positive Tangentenpunkte
    xpos = [x[0] for x in TPp]
    ypos = [x[1] for x in TPp]
    #Negative Tangentenpunkte
    xneg = [x[0] for x in TPn]
    yneg = [x[1] for x in TPn]
    #Keine Ahnung
    cmap = plt.cm.hot
    fig = plt.figure()
    ax = fig.add_subplot(111, aspect='equal')
    #Hier werden die Rechtecke an die Punkte geklebt color=cmap(0.55)
    for x, y in zip(xpunkte, ypunkte):
        ax.add_artist(plt.Circle((x + R , y), R, color=cmap(0.55)))
        ax.add_artist(plt.Circle( ( x + R , y ), R , fill = False ))
    #Plotte alles
    plt.plot(xpunkte,ypunkte,'or')
    plt.plot(xpos,ypos,'og')
    plt.plot(xneg,yneg,'ob')
    plt.axis([0, a, 0, b])
    print(len(TPp)-len(TPn))
    plt.show()
    






#Histogramplot
def Histogram(n,gamma,a,b,R):    
    #Generiere Daten und plotte sie in ein Histogram
    data = [Chi(gamma,a,b,R) for x in np.empty(n)]
    plt.hist(data, normed=True, bins=15)
    #Plotbereich
    xt = plt.xticks()[0]  
    xmin, xmax = min(xt), max(xt)  
    lnspc = np.linspace(xmin, xmax, n)
    #Standardnormalverteilung
    pdf_g = stats.norm.pdf(lnspc, 0, 1)
    plt.plot(lnspc, pdf_g, label="Norm")    
    plt.show()  


