import numpy as np
from numpy.linalg import norm
from math import pi, sqrt,exp
import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle
from scipy import stats
import matplotlib.pyplot as plt




#Erzeuge Poisson-Prozess
def Punkte(gamma,a,b):
    #Beachte: die "+-1" bzw. "2" sind der Einfluss von ausserhalb des Fensters
    Z = np.random.uniform(low=(-2,-1), high=(a,b+1), size=(np.random.poisson(gamma*(a+2)*(b+2)),2))
    #Gib die lexikographisch geordnete Liste aus
    return(Z[np.lexsort(np.rot90(Z))])



#Pruefe ob positive TP ueberdeckt werden
def TestPos(Z,R,a,b):
    N = len(Z)
    Loeschen = []
    for i in range(N-1):
        j = i + 1 
        while j <= N-1 and Z[j][0]-Z[i][0] < 2:
            if norm(Z[j] + [R[j],-R[j]] - (Z[i] + [R[i],0]),2) < R[i]:
                #Falls ja wird der j-te Tangentenpunkt überdeckt
                Loeschen.append(j)
            if norm(Z[i] + [R[i],-R[i]] - (Z[j] + [R[j],0]),2) < R[j]:
                #Falls ja wird der i-te Tangentenpunkt überdeckt
                Loeschen.append(i)
            j = j+1
    Radius = [[r,-r] for r in R]
    TP = np.delete(Z + Radius,Loeschen,0)
    return(np.delete(TP,[i for i,x in enumerate(TP) if x[0] < 0 or x[0] > a or x[1] < 0 or x[1] > b],0))






#Sichtbare negative Tangentenpunkte bestimmen
def TestNeg(Z,R,a,b):
    N = len(Z)
    TP = []
    #Finde mögliche TP    
    for i in range(N-1):
        j = i + 1 
        while j <= N-1 and Z[j][0]-Z[i][0] < 2:
            #Dazu ist die notwendige Bedigung, dass die beiden Kreise sich schneiden
            c = norm(Z[j]+[R[j],0] - (Z[i]+[R[i],0]) ,2)
            if c < R[i]+R[j] and c > abs(R[i]-R[j]):
                if Z[j][0] + R[j] > Z[i][0]+R[i]:
                    #Welche Koordinaten hat dann der moegliche TP?
                    x = (R[i]**2 + c**2 - R[j]**2)/(2*c)
                    TPneg = Z[i]+np.array([R[i],0]) + x/c*(Z[j]+np.array([R[j],0]) - (Z[i]+np.array([R[i],0]))) - ( R[i]**2 - x**2 )**(1/2)/c *np.array([Z[i][1]-Z[j][1] , Z[j][0]+R[j]-(Z[i][0]+R[i])])
                    if (TPneg[0] > Z[i][0] + R[i] and TPneg[0] < Z[j][0] + R[j]) or (TPneg[0] < Z[i][0] + R[i] and TPneg[0] > Z[j][0] + R[j]):
                        #Wird dieser TP ueberdeckt?
                        k = 0
                        while k <= N-1 and Z[k][0]-TPneg[0] < 2:
                            if k != j and k != i and norm(TPneg-Z[k]-[R[k],0],2) < R[k]:
                                k = N + 3
                            else:
                                k = k + 1
                        if k <= N+1:
                            TP.append(TPneg)
                else:
                    #Welche Koordinaten hat dann der moegliche TP?
                    c = norm(Z[i]+[R[i],0] - (Z[j]+[R[j],0]) ,2)
                    x = (R[j]**2 + c**2 - R[i]**2)/(2*c)
                    TPneg = Z[j]+np.array([R[j],0]) + x/c*(Z[i]+np.array([R[i],0]) - (Z[j]+np.array([R[j],0]))) - ( R[j]**2 - x**2 )**(1/2)/c *np.array([Z[j][1]-Z[i][1] , Z[i][0]+R[i]-(Z[j][0]+R[j])])
                    if (TPneg[0] > Z[j][0] + R[j] and TPneg[0] < Z[i][0] + R[i]) or (TPneg[0] < Z[j][0] + R[j] and TPneg[0] > Z[i][0] + R[i]):
                        #Wird dieser TP ueberdeckt?
                        k = 0
                        while k <= N-1 and Z[k][0]-TPneg[0] < 2:
                            if k != j and k != i and norm(TPneg-Z[k]-[R[k],0],2) < R[k]:
                                k = N + 3
                            else:
                                k = k + 1
                        if k <= N+1:
                            TP.append(TPneg)
            j = j+1
    return(np.delete(TP,[i for i,x in enumerate(TP) if x[0] < 0 or x[0] > a or x[1] < 0 or x[1] > b],0))





#Definiere die Kernfunktion
def K(x):
    z = norm(x,2)
    if z < 1:
        Wert = 1-z**2
    else:
        Wert = 0
    return(Wert)


#Definiere die Funktion zum Flaecheberechnen
def Area(u):
    return(u[0]*u[1])



#Berechne die EPC
def Chi(gamma,a,b):
    Z = Punkte(gamma,a,b)
    R = np.random.uniform(0, 1, len(Z))
    TPp = TestPos(Z,R,a,b)
    TPn = TestNeg(Z,R,a,b)
    Chi = len(TPp) - len(TPn)
    folge = (a*b)**(1/6)
    sigma = sum([K((x-y)/folge)/Area([a,b]-abs(x-y)) for x in TPp for y in TPp]) + sum([K((x-y)/folge)/Area([a,b]-abs(x-y)) for x in TPn for y in TPn]) - 2*sum([K((x-y)/folge)/Area([a,b]-abs(x-y)) for x in TPp for y in TPn]) - folge**2*pi/2*(Chi/(a*b))**2 
    Normal = sqrt(a*b/sigma)*(Chi/(a*b)-(gamma*exp(-gamma*pi/3)*(1-gamma*pi/4)))
    return(Normal)








#Testplot
def Testplot(gamma,a,b):
    #Erstelle Liste mit Punkten
    Z = Punkte(gamma,a,b)
    R = np.random.uniform(0, 1, len(Z))
    TPp = TestPos(Z,R,a,b)
    TPn = TestNeg(Z,R,a,b)
    #Fummle die Liste auseinander
    xpunkte = [x[0] for x in Z]
    ypunkte = [x[1] for x in Z]
    #Positive Tangentenpunkte
    xpos = [x[0] for x in TPp]
    ypos = [x[1] for x in TPp]
    #Negative Tangentenpunkte
    xneg = [x[0] for x in TPn]
    yneg = [x[1] for x in TPn]
    #Keine Ahnung
    cmap = plt.cm.hot
    fig = plt.figure()
    ax = fig.add_subplot(111, aspect='equal')
    #Hier werden die Rechtecke an die Punkte geklebt color=cmap(0.55)
    for x, y, r in zip(xpunkte, ypunkte, R):
        ax.add_artist(plt.Circle((x + r , y), r, color=(0.30,0.62,0.62)))
    #Plotte alles
    #plt.plot(xpunkte,ypunkte,'or')
    plt.plot(xpos,ypos,'ob')
    plt.plot(xneg,yneg,'or')
    plt.axis([0, a, 0, b])
    print(len(TPp)-len(TPn))
    plt.show()
    

