import numpy as np
from numpy.linalg import norm
from math import pi, sqrt,exp
import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle
from scipy import stats
import matplotlib.pyplot as plt




#Erzeuge Poisson-Prozess
def Punkte(gamma,a,b):
    #Beachte: die "+1" ist der Einfluss von ausserhalb des Fensters
    Z = np.random.uniform(low = (-1,-1), high=(a,b), size=(np.random.poisson(gamma*(a+1)*(b+1)),2))
    #Gib die lexikographisch geordnete Liste aus
    return(Z[np.lexsort(np.rot90(Z))])
                                




def TestPos(Z):
    #Pruefe ob positive TP ueberdeckt werden
    
    #Langsamer
    #TP = np.delete(Z,[i for i,a in enumerate(Z) for b in Z if a[0]-b[0] < 1 and a[0]-b[0] > 0 and a[1]-b[1] > 0 and a[1]-b[1] < 1],0)

    #Schneller
    N = len(Z)
    Loeschen = []
    for i in range(N-1):
        j = i + 1 
        while j <= N-1 and Z[j][0]-Z[i][0] < 1:
            if 0 < Z[j][1] - Z[i][1] and Z[j][1] - Z[i][1] < 1:
                #Falls ja wird der Tangentenpunkt überdeckt
                Loeschen.append(j)
            j = j+1
    TP = np.delete(Z,Loeschen,0)


    #Loesche die positiven TP ausserhalb des Fensters

    #Langsamer
    #Loeschen = []
    #for i in range(len(TP)):
    #    if TP[i][0] < 0 or TP[i][1] < 0:
    #        Loeschen.append(i)
    #return(np.delete(TP,Loeschen,0))

    #Schneller
    return(np.delete(TP,[i for i,t in enumerate(TP) if t[0] < 0 or t[1] < 0],0))





#Sichtbare negative Tangentenpunkte bestimmen
def TestNeg(Z):
    N = len(Z)
    TPneg = []
    #Finde mögliche TP
    for i in range(N-1):
        j = i + 1 
        while j <= N-1 and Z[j][0]-Z[i][0] < 1:
            if -1 < Z[j][1] - Z[i][1] and Z[j][1] - Z[i][1] < 0:
                TPneg.append([Z[j][0],Z[i][1]])
            j = j+1


    #Werden diese TP ueberdeckt?
    #Loeschen = []
    #for i in range(N-1):
    #    for j in range(len(TPneg)):
    #        if 0 < TPneg[j][0] - Z[i][0] and TPneg[j][0] - Z[i][0] < 1:
    #            if 0 < TPneg[j][1] - Z[i][1] and TPneg[j][1] - Z[i][1] < 1:
    #                Loeschen.append(j)
    #        j = j+1
    #TP = np.delete(TPneg,Loeschen,0)

    TP = np.delete(TPneg,[i for i,t in enumerate(TPneg) for z in Z if t[0]-z[0] > 0 and t[0]-z[0] < 1 and t[1]-z[1] > 0 and t[1]-z[1] < 1],0)


    #Loeschen = []
    #for i in range(len(TP)):
    #    if TP[i][0] < 0 or TP[i][1] < 0:
    #        Loeschen.append(i)            
    #return(np.delete(TP,Loeschen,0))
    return(np.delete(TP,[i for i,t in enumerate(TP) if t[0] < 0 or t[1] < 0],0))



#Definiere die Kernfunktion
def K(x):
    z = norm(x,2)
    if z < 1:
        Wert = 1-z**2
    else:
        Wert = 0
    return(Wert)


#Definiere die Funktion zum Flaecheberechnen
def Area(u):
    return(u[0]*u[1])



#Berechne die EPC
def Chi(gamma,a,b):
    Z = Punkte(gamma,a,b)
    N = len(Z)
    Loeschen = []
    TPneg = []
    for i in range(N-1):
        j = i + 1 
        while j <= N-1 and Z[j][0]-Z[i][0] < 1:
            if 0 < Z[j][1] - Z[i][1] and Z[j][1] - Z[i][1] < 1:
                #Falls ja wird der Tangentenpunkt überdeckt
                Loeschen.append(j)
            elif -1 < Z[j][1] - Z[i][1] and Z[j][1] - Z[i][1] < 0:
                #Falls ja hat man moeglicherweise einen negativen TP
                TPneg.append([Z[j][0],Z[i][1]])
            j = j + 1
    TPp = np.delete(Z,Loeschen,0)



    #Loeschen = []
    #for i in range(len(TPp)):
    #    if TPp[i][0] < 0 or TPp[i][1] < 0:
    #        Loeschen.append(i)
    #TPp = np.delete(TPp,Loeschen,0)
    TPp = np.delete(TPp,[i for i,t in enumerate(TPp) if t[0] < 0 or t[1] < 0],0)


    #Werden die negativen TP ueberdeckt?
    #Loeschen = []
    #for i in range(N-1):
    #    for j in range(len(TPneg)):
    #        if 0 < TPneg[j][0] - Z[i][0] and TPneg[j][0] - Z[i][0] < 1:
    #            if 0 < TPneg[j][1] - Z[i][1] and TPneg[j][1] - Z[i][1] < 1:
    #                Loeschen.append(j)
    #        j = j+1
    #TPn = np.delete(TPneg,Loeschen,0)


    TPn = np.delete(TPneg,[i for i,t in enumerate(TPneg) for z in Z if t[0]-z[0] > 0 and t[0]-z[0] < 1 and t[1]-z[1] > 0 and t[1]-z[1] < 1],0)


    #Loeschen = []
    #for i in range(len(TPn)):
    #    if TPn[i][0] < 0 or TPn[i][1] < 0:
    #        Loeschen.append(i)
    #TPn = np.delete(TPn,Loeschen,0)
    TPn = np.delete(TPn,[i for i,t in enumerate(TPn) if t[0] < 0 or t[1] < 0],0)


    #Berechne gesuchte Zahl
    Chi = len(TPp) - len(TPn)    
    folge = (a*b)**(1/6)
    sigma = sum([K((x-y)/folge)/Area([a,b]-abs(x-y)) for x in TPp for y in TPp]) + sum([K((x-y)/folge)/Area([a,b]-abs(x-y))  for x in TPn for y in TPn]) - 2*sum([K((x-y)/folge)/Area([a,b]-abs(x-y))  for x in TPp for y in TPn]) - folge**2*pi/2*(Chi/(a*b))**2 
    Normal = sqrt(a*b/sigma)*(Chi/(a*b)-(gamma*exp(-gamma)-gamma**2*exp(-gamma)/2*(4-2)))
    return(Normal)
































#Testplot
def Testplot(gamma,a,b):
    #Erstelle Liste mit Punkten
    Z = Punkte(gamma,a,b)
    TPp = TestPos(Z)
    TPn = TestNeg(Z)
    #Fummle die Liste auseinander
    xpunkte = [x[0] for x in Z]
    ypunkte = [x[1] for x in Z]
    #Positive Tangentenpunkte
    xpos = [x[0] for x in TPp]
    ypos = [x[1] for x in TPp]
    #Negative Tangentenpunkte
    xneg = [x[0] for x in TPn]
    yneg = [x[1] for x in TPn]
    #Keine Ahnung
    cmap = plt.cm.hot
    fig = plt.figure()
    ax = fig.add_subplot(111, aspect='equal')
    #Hier werden die Rechtecke an die Punkte geklebt
    for x, y in zip(xpunkte, ypunkte):
        ax.add_artist(Rectangle(xy=(x, y), color=cmap(0.55), width=1, height=1))
    #Plotte alles
    plt.plot(xpunkte,ypunkte,'or')
    plt.plot(xpos,ypos,'og')
    plt.plot(xneg,yneg,'ob')
    plt.axis([0, a, 0, b])
    print(len(TPp)-len(TPn))
    plt.show()






#Histogramplot
def Histogram(n,gamma,a,b):    
    #Generiere Daten und plotte sie in ein Histogram
    data = [Chi(gamma,a,b) for x in np.empty(n)]
    plt.hist(data, normed=True, bins=15)
    #Plotbereich
    xt = plt.xticks()[0]  
    xmin, xmax = min(xt), max(xt)  
    lnspc = np.linspace(xmin, xmax, n)
    #Standardnormalverteilung
    pdf_g = stats.norm.pdf(lnspc, 0, 1)
    plt.plot(lnspc, pdf_g, label="Norm")    
    plt.show()  

