import numpy as np
from numpy.linalg import norm
from math import pi, sqrt,exp
import matplotlib.pyplot as plt
from matplotlib.patches import Rectangle
from scipy import stats
import matplotlib.pyplot as plt




#Erzeuge Poisson-Prozess
def Punkte(gamma,a,b):
    #Beachte: die "+1" ist der Einfluss von ausserhalb des Fensters
    Z = np.random.uniform(low = (-1,-1), high=(a,b), size=(np.random.poisson(gamma*(a+1)*(b+1)),2))
    #Gib die lexikographisch geordnete Liste aus
    return(Z[np.lexsort(np.rot90(Z))])
                                



#Pruefe ob positive TP ueberdeckt werden
def TestPos(Z,A,B):
    N = len(Z)
    Loeschen = []
    for i in range(N-1):
        j = i + 1 
        while j <= N-1 and Z[j][0] < Z[i][0] + A[i]:
            if 0 < Z[j][1] - Z[i][1] and Z[j][1] < Z[i][1] + B[i]:
                #Falls ja wird der Tangentenpunkt überdeckt
                Loeschen.append(j)
            j = j+1
    TP = np.delete(Z,Loeschen,0)
    return(np.delete(TP,[i for i,t in enumerate(TP) if t[0] < 0 or t[1] < 0],0))





#Sichtbare negative Tangentenpunkte bestimmen
def TestNeg(Z,A,B):
    N = len(Z)
    TPneg = []
    #Finde mögliche TP
    for i in range(N-1):
        j = i + 1 
        while j <= N-1 and Z[j][0] < Z[i][0] + A[i]:
            if Z[i][1] < Z[j][1] + B[j]  and Z[j][1] < Z[i][1]:
                TPneg.append([Z[j][0],Z[i][1]])
            j = j+1
    #Werden diese TP ueberdeckt?
    #Loeschen = []
    #for i in range(N-1):
    #    for j in range(len(TPneg)):
    #        if 0 < TPneg[j][0] - Z[i][0] and TPneg[j][0] - Z[i][0] < 1:
    #            if 0 < TPneg[j][1] - Z[i][1] and TPneg[j][1] - Z[i][1] < 1:
    #                Loeschen.append(j)
    #        j = j+1
    #TP = np.delete(TPneg,Loeschen,0)
    TP = np.delete(TPneg,[i for i,t in enumerate(TPneg) for z,w,h in zip(Z,A,B) if t[0]-z[0] > 0 and t[0]-z[0] < w and t[1]-z[1] > 0 and t[1]-z[1] < h],0)
    #Loeschen = []
    #for i in range(len(TP)):
    #    if TP[i][0] < 0 or TP[i][1] < 0:
    #        Loeschen.append(i)            
    #return(np.delete(TP,Loeschen,0))
    return(np.delete(TP,[i for i,t in enumerate(TP) if t[0] < 0 or t[1] < 0],0))



#Definiere die Kernfunktion
def K(x):
    z = norm(x,2)
    if z < 1:
        Wert = 1-z**2
    else:
        Wert = 0
    return(Wert)


#Definiere die Funktion zum Flaecheberechnen
def Area(u):
    return(u[0]*u[1])



#Berechne die EPC
def Chi(gamma,a,b):
    Z = Punkte(gamma,a,b)
    N = len(Z)
    A = np.random.uniform(0, 1, N)
    B = np.random.uniform(0, 1, N)
    TPp = TestPos(Z,A,B)
    TPn = TestNeg(Z,A,B)
    Chi = len(TPp) - len(TPn)    
    folge = (a*b)**(1/6)
    sigma = sum([K((x-y)/folge)/Area([a,b]-abs(x-y)) for x in TPp for y in TPp]) + sum([K((x-y)/folge)/Area([a,b]-abs(x-y))  for x in TPn for y in TPn]) - 2*sum([K((x-y)/folge)/Area([a,b]-abs(x-y))  for x in TPp for y in TPn]) - folge**2*pi/2*(Chi/(a*b))**2 
    Normal = sqrt(a*b/sigma)*(Chi/(a*b)-(gamma*exp(-gamma/4)-gamma**2*exp(-gamma/4)/4))
    return(Normal)







#Testplot
def Testplot(gamma,a,b):
    #Erstelle Liste mit Punkten
    Z = Punkte(gamma,a,b)
    A = np.random.uniform(0, 1, len(Z))
    B = np.random.uniform(0, 1, len(Z))
    TPp = TestPos(Z,A,B)
    TPn = TestNeg(Z,A,B)
    #Fummle die Liste auseinander
    xpunkte = [x[0] for x in Z]
    ypunkte = [x[1] for x in Z]
    #Positive Tangentenpunkte
    xpos = [x[0] for x in TPp]
    ypos = [x[1] for x in TPp]
    #Negative Tangentenpunkte
    xneg = [x[0] for x in TPn]
    yneg = [x[1] for x in TPn]
    #Keine Ahnung
    cmap = plt.cm.hot
    fig = plt.figure()
    ax = fig.add_subplot(111, aspect='equal')
    #Hier werden die Rechtecke an die Punkte geklebt
    for x,y,w,h in zip(xpunkte, ypunkte, A, B):
        ax.add_artist(Rectangle(xy=(x, y), color=(0.30,0.62,0.62), width=w, height=h))
    #Plotte alles
    #plt.plot(xpunkte,ypunkte,'or')
    plt.plot(xpos,ypos,'ob')
    plt.plot(xneg,yneg,'or')
    plt.axis([0, a, 0, b])
    print(len(TPp)-len(TPn))
    plt.show()



